package cz.mendelu.dnaAnalyser.analyse.palindrome.detector;

import cz.mendelu.dnaAnalyser.analyse.palindrome.model.PalindromeModel;

import cz.mendelu.dnaAnalyser.sequence.NucleicBuffer;
import cz.mendelu.dnaAnalyser.sequence.data.SequenceData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Stream;

/**
 * Created by xkoloma1 on 27. 7. 2015.
 */
public class SimplePalindromeDetector implements PalindromeDetector {

    private static final Logger LOG = LoggerFactory.getLogger(SimplePalindromeDetector.class);

    private final int size;

    private final int spacer;

    private final int mismatches;

    public SimplePalindromeDetector(int size, int spacer, int mismatches) {
        LOG.debug("Create: size = {}, spacer = {}, mismatches = {}", size, spacer, mismatches);
        this.size = size;
        this.spacer = spacer;
        this.mismatches = mismatches;
    }

    @Override
    public PalindromeMatcher findPalindrome(SequenceData sequence) {
        SimplePalindromeMatcher palindromeMatcher = new SimplePalindromeMatcher();
        Integer mismatche;
        NucleicBuffer testedSequence, oppositeSequence, nextSequence;
        if(sequence.getLength() >= (size + spacer + size)) {
            for (int i = 0; i < sequence.getLength() - (size + spacer + size - 1); i++) {
                testedSequence = sequence.getBuffer(i, size);
                oppositeSequence = testedSequence.revert();
                nextSequence = sequence.getBuffer(i + size + spacer, size);
                mismatche = oppositeSequence.mismatches(nextSequence);
                if (mismatche != null && mismatche == mismatches) {
                    palindromeMatcher.add(new PalindromeModel(sequence, i, size, spacer, mismatches));
                }
            }
        }
        LOG.info("AnalysePalindrome done, found: {}", palindromeMatcher.getCount());
        return palindromeMatcher;
    }

    @Override
    public void setAtatFilter(boolean atatFilter) {
        // Ignore
    }

    public static class SimplePalindromeMatcher implements PalindromeMatcher {

        private List<PalindromeModel> palindromeModels = new LinkedList<>();

        void add(PalindromeModel palindromeModel) {
            palindromeModels.add(palindromeModel);
        }

        @Override
        public int getCount() {
            return palindromeModels.size();
        }

        @Override
        public Stream<PalindromeModel> stream() {
            return palindromeModels.stream();
        }

        @Override
        public Iterator<PalindromeModel> iterator() {
            return palindromeModels.iterator();
        }
    }
}
