package cz.mendelu.dnaAnalyser.utils.service;

import cz.mendelu.dnaAnalyser.utils.exception.NotFoundException;
import cz.mendelu.dnaAnalyser.utils.repository.BaseRepository;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Set;
import java.util.UUID;

public abstract class ModelService<M> {

    protected abstract BaseRepository<M, UUID> getRepository();

    Class<M> modelClass() {
        Type type = getClass().getGenericSuperclass();
        ParameterizedType paramType = (ParameterizedType) type;
        return (Class<M>) paramType.getActualTypeArguments()[0];
    }

    public Page<M> findAll(Pageable pageable) {
        BaseRepository<M, UUID> baseRepository = getRepository();
        return baseRepository.findAll(pageable);
    }

    public Page<M> findAll(Pageable pageable, Set<String> tags) {
        BaseRepository<M, UUID> baseRepository = getRepository();
        String[] tagsArray = (tags != null)
                ? tags.toArray(new String[tags.size()])
                : new String[0];

        switch (tagsArray.length) {
            case 0:
                return baseRepository.findAll(pageable);
            case 1:
                return baseRepository.findAll(pageable, tagsArray[0]);
            default:
                throw new IllegalArgumentException("Maximum tags is 1.");
        }
    }

    public M findOne(UUID id) {
        BaseRepository<M, UUID> baseRepository = getRepository();
        return baseRepository.findById(id).orElseThrow(
                () -> new NotFoundException(modelClass(), id)
        );
    }

    public M save(M modelEntity) {
        return getRepository().save(modelEntity);
    }

    public void delete(UUID id) {
        BaseRepository<M, UUID> baseRepository = getRepository();
        M one = baseRepository.findById(id).orElseThrow(
                () -> new NotFoundException(modelClass(), id)
        );
        getRepository().delete(one);
    }

    public Page<String> findAllTags(Pageable pageable) {
        BaseRepository<M, UUID> baseRepository = getRepository();
        Page<String> tagsPage = baseRepository.findAllTags(pageable);
        return tagsPage;
    }
}
